package io.flutter.plugins.webviewflutter;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.pm.ActivityInfo;
import android.media.MediaPlayer;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.WebChromeClient;
import android.widget.FrameLayout;
import android.widget.VideoView;

/**
 * Created on 05.26 - 2020
 *
 * @author wenbosheng
 * Copyright (c) 2020 android Amber
 */
public class VideoViewController {

    private View customView;
    private VideoView videoView;
    private Activity holderActivity;
    private int originalVisibility;
    private int originalOrientation;
    private WebChromeClient.CustomViewCallback customViewCallback;

    public VideoViewController() {
    }

    public void setHolderActivity(Activity holderActivity) {
        this.holderActivity = holderActivity;
    }

    @SuppressLint("SourceLockedOrientationActivity")
    void onShowCustomView(final View view, final WebChromeClient.CustomViewCallback callback) {
        if (view == null) {
            return;
        }
        if (customView != null && callback != null) {
            callback.onCustomViewHidden();
            return;
        }
        final Activity activity = holderActivity;
        if (activity == null) {
            return;
        }
        originalOrientation = activity.getRequestedOrientation();
        customView = view;
        customViewCallback = callback;
        FrameLayout decorView = (FrameLayout) activity.getWindow().getDecorView();
        view.setBackgroundColor(0xff000000);
        originalVisibility = decorView.getSystemUiVisibility();
        decorView.addView(customView, new FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
        decorView.setSystemUiVisibility(
                View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY
                        // Set the content to appear under the system bars so that the
                        // content doesn't resize when the system bars hide and show.
                        | View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                        | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                        | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                        // Hide the nav bar and status bar
                        | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                        | View.SYSTEM_UI_FLAG_FULLSCREEN);

        customView.setKeepScreenOn(true);
        if (view instanceof FrameLayout) {
            if (((FrameLayout) view).getFocusedChild() instanceof VideoView) {
                videoView = (VideoView) ((FrameLayout) view).getFocusedChild();
                videoView.setOnErrorListener(new VideoCompletionListener());
                videoView.setOnCompletionListener(new VideoCompletionListener());
            }
        }
        activity.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
    }

    boolean onHideCustomView() {
        if (customView == null || customViewCallback == null) {
            return false;
        }
        customView.setKeepScreenOn(false);
        final Activity activity = holderActivity;
        if (activity != null) {
            FrameLayout decorView = (FrameLayout) activity.getWindow().getDecorView();
            decorView.removeView(customView);
            decorView.setSystemUiVisibility(originalVisibility);
        }
        customView = null;
        if (videoView != null) {
            videoView.setOnErrorListener(null);
            videoView.setOnCompletionListener(null);
            videoView = null;
        }
        if (activity!=null) {
            activity.setRequestedOrientation(originalOrientation);
        }
        return true;
    }


    private class VideoCompletionListener implements MediaPlayer.OnCompletionListener, MediaPlayer.OnErrorListener {
        @Override
        public boolean onError(MediaPlayer mp, int what, int extra) {
            return false;
        }

        @Override
        public void onCompletion(MediaPlayer mp) {
            onHideCustomView();
        }
    }

}
